@@ -3,6 +3,7 @@ from labelbase import Client as labelbaseClient
 from labelbox.schema.dataset import Dataset as labelboxDataset
 import pandas as pd
 from labelpandas import connector
+from concurrent.futures import ThreadPoolExecutor, as_completed
 
 class Client():
     """
@@ -28,21 +29,11 @@ class Client():
     # def create_table_from_dataset(): 
     #     return table 
 
-    def create_data_rows_from_table(
-        self,
-        table:pd.core.frame.DataFrame,
-        lb_dataset:labelboxDataset, 
-        row_data_col:str, 
-        local_files:bool=False,
-        global_key_col:str="", 
-        external_id_col:str="", 
-        metadata_index:dict={}, 
-        skip_duplicates:bool=False,
-        divider="___",
-        verbose:bool=False):
+    def create_data_rows_from_table(self, df:pd.core.frame.DataFrame, lb_dataset:labelboxDataset, row_data_col:str, local_files:bool=False,
+        global_key_col=None, external_id_col=None, metadata_index:dict={}, skip_duplicates:bool=False, divider="___", verbose:bool=False):
         """ Creates Labelbox data rows given a Pandas table and a Labelbox Dataset
         Args:
-            table           :   Required (pandas.core.frame.DataFrame) - Pandas dataframe to-be-uploaded
+            df              :   Required (pandas.core.frame.DataFrame) - Pandas dataframe to-be-uploaded
             lb_dataset      :   Required (labelbox.schema.dataset.Dataset) - Labelbox dataset to add data rows to            
             row_data_col    :   Required (str) - Column name where the data row row data URL is located
             local_files     :   Required (bool) - If True, will create urls for local files / If False, treats the values in `row_data_col` as urls
@@ -54,54 +45,30 @@ class Client():
             verbose         :   Required (bool) - If True, prints information about code execution
         Returns:
             List of errors from data row upload - if successful, is an empty list
-        """        
-        check = self.base_client.enforce_metadata_index(
-            metadata_index=metadata_index, 
-            verbose=verbose
-        )
-        if not check:
-            return None
-        table = self.base_client.sync_metadata_fields(
-            table=table, 
-            get_columns_function=connector.get_columns_function, 
-            add_column_function=connector.add_column_function, 
-            get_unique_values_function=connector.get_unique_values_function, 
-            metadata_index=metadata_index, 
-            verbose=verbose
-        )
-        if not table:
-            return None
+        """
         
-        global_key_col = global_key_col if global_key_col else row_data_col
-        external_id_col = external_id_col if external_id_col else global_key_col
-
-        metadata_schema_to_name_key = self.base_client.get_metadata_schema_to_name_key(
-            lb_mdo=False, 
-            divider=divider, 
-            invert=False
+        # Ensure all your metadata_index keys are metadata fields in Labelbox and that your Pandas DataFrame has all the right columns
+        df = self.base_client.sync_metadata_fields(
+            table=df, get_columns_function=connector.get_columns_function, add_column_function=connector.add_column_function, 
+            get_unique_values_function=connector.get_unique_values_function, metadata_index=metadata_index, verbose=verbose
         )
-        metadata_name_key_to_schema = self.base_client.get_metadata_schema_to_name_key(
-            lb_mdo=False, 
-            divider=divider, 
-            invert=True
+        
+        # If df returns False, the sync failed
+        if type(df) == bool:
+            return None   
+        
+        # Create a dictionary where {key=global_key : value=labelbox_upload_dictionary} - this is unique to Pandas
+        global_key_to_upload_dict = connector.create_upload_dict(
+            df=df, local_files=local_files, lb_client=self.lb_client,
+            row_data_col=row_data_col, global_key_col=global_key_col, 
+            external_id_col=external_id_col, metadata_index=metadata_index, divider=divider
+        )
+                
+        # Upload your data rows to Labelbox
+        upload_results = self.base_client.batch_create_data_rows(
+            dataset=lb_dataset, global_key_to_upload_dict=global_key_to_upload_dict, 
+            skip_duplicates=skip_duplicates, divider=divider
         )
-
-        global_key_to_upload_dict = {}
-        futures = []
-        with ThreadPoolExecutor() as exc:
-            for index, row in table.iterrows():
-                futures.append(
-                    exc.submit(
-                        connector.create_data_rows, local_files, self.lb_client, row, row_data_col, 
-                        global_key_col, external_id_col, metadata_index, metadata_name_key_to_schema,
-                        metadata_schema_to_name_key, divider
-                    )
-                )
-            for f in as_completed(futures):
-                res = f.result()
-                global_key_to_upload_dict[str(res["global_key"])] = res
-
-        upload_results = self.base_client.batch_create_data_rows(lb_dataset, global_key_to_upload_dict, skip_duplicates, divider)
 
         return upload_results
     
